@@ -18,13 +18,24 @@ Geo::Spline - Calculate geographic locations between GPS fixes.
          speed=>30.615,
          heading=>150.5300};
  my $spline=Geo::Spline->new($p0, $p1);
- my $pt=$spline->time(1160449150);
+ my $point=$spline->point(1160449150);
  print "Lon", $pt->{"lat"}, "Lat", $pt->{"lon"}, "\n";
- my @point=$spline->(); #default is int(t2-t1+.5)
+ my $pointlistref=$spline->pointlist(); #default is int(t2-t1+.5)
 
 =head1 DESCRIPTION
 
-This module is a pure Perl port of the NGS program in the public domain "forward" by Robert (Sid) Safford and Stephen J. Frakes.  
+This program was developed to be able to calculate the position between two GPS fixes using a 2-dimintional 3rd order polynominal spline.
+
+ f(t)  = A + B(t-t0)  + C(t-t0)^2 + D(t-t0)^3 #position in X and Y
+ f'(t) = B + 2C(t-t0) + 3D(t-t0)^2            #velocity in X and Y
+
+I did some simplae Math (for an engineer with a math minor) to come up with these formulas to calculate the unknowns from our knowns.
+
+ A = x0                                     # when (t-t0)=0 in f(t)
+ B = v0                                     # when (t-t0)=0 in f'(t)
+ C = (x1-A-B(t1-t0)-D(t1-t0)^3)/(t1-t0)^2   # solve for C from f(t)
+ C = (v1-B-3D(t1-t0)^2)/2(t1-t0)            # solve for C from f'(t)
+ D = (v1(t1-t0)+B(t1-t0)-2x1+2A)/(t1-t0)^3  # equate C=C then solve for D
 
 =cut
 
@@ -37,7 +48,7 @@ use constant earth_equatorial_circumference_meters_per_degree => 6378137 * PI/18
 use constant EPCMPD => earth_polar_circumference_meters_per_degree;
 use constant EECMPD => earth_equatorial_circumference_meters_per_degree;
 
-$VERSION = sprintf("%d.%02d", q{Revision: 0.03} =~ /(\d+)\.(\d+)/);
+$VERSION = sprintf("%d.%02d", q{Revision: 0.06} =~ /(\d+)\.(\d+)/);
 
 =head1 METHODS
 
@@ -54,28 +65,28 @@ sub new {
 
 sub initialize {
   my $self = shift();
+  $self->{'pt0'}=shift();
   $self->{'pt1'}=shift();
-  $self->{'pt2'}=shift();
-  my $dt=$self->{'pt2'}->{'time'} - $self->{'pt1'}->{'time'};
-  die ("Delta time is must be greater than 0") if ($dt<=0);
+  my $dt=$self->{'pt1'}->{'time'} - $self->{'pt0'}->{'time'};
+  die ("Delta time must be greater than zero.") if ($dt<=0);
   my ($A, $B, $C, $D)=$self->ABCD(
+     $self->{'pt0'}->{'time'},
+     $self->{'pt0'}->{'lat'} * EPCMPD,
+     $self->{'pt0'}->{'speed'} * cos($self->{'pt0'}->{'heading'}/RAD),
      $self->{'pt1'}->{'time'},
      $self->{'pt1'}->{'lat'} * EPCMPD,
-     $self->{'pt1'}->{'speed'} * cos($self->{'pt1'}->{'heading'}/RAD),
-     $self->{'pt2'}->{'time'},
-     $self->{'pt2'}->{'lat'} * EPCMPD,
-     $self->{'pt2'}->{'speed'} * cos($self->{'pt2'}->{'heading'}/RAD));
+     $self->{'pt1'}->{'speed'} * cos($self->{'pt1'}->{'heading'}/RAD));
   $self->{'Alat'}=$A;
   $self->{'Blat'}=$B;
   $self->{'Clat'}=$C;
   $self->{'Dlat'}=$D;
   ($A, $B, $C, $D)=$self->ABCD(
+     $self->{'pt0'}->{'time'},
+     $self->{'pt0'}->{'lon'} * EECMPD,
+     $self->{'pt0'}->{'speed'} * sin($self->{'pt0'}->{'heading'}/RAD),
      $self->{'pt1'}->{'time'},
      $self->{'pt1'}->{'lon'} * EECMPD,
-     $self->{'pt1'}->{'speed'} * sin($self->{'pt1'}->{'heading'}/RAD),
-     $self->{'pt2'}->{'time'},
-     $self->{'pt2'}->{'lon'} * EECMPD,
-     $self->{'pt2'}->{'speed'} * sin($self->{'pt2'}->{'heading'}/RAD));
+     $self->{'pt1'}->{'speed'} * sin($self->{'pt1'}->{'heading'}/RAD));
   $self->{'Alon'}=$A;
   $self->{'Blon'}=$B;
   $self->{'Clon'}=$C;
@@ -92,8 +103,8 @@ sub ABCD {
   my $v1 = shift();
   #x=f(t)=A+B(t-t0)+C(t-t0)^2+D(t-t0)^3
   #v=f'(t)=B+2C(t-t0)+3D(t-t0)^2
-  #A=x0-B(t0-t0)=C(t0-t0)^2+D(t0-t0)^3
-  #B=v0-2C(t0-t0)-3D(t0-t0)^2
+  #A=x0
+  #B=v0
   #C=(x1-A-B(t1-t0)-D(t1-t0)^3)/((t1-t0)^2) # from f(t)
   #C=(v1-B-3D(t1-t0)^2)/2(t1-t0)            # from f'(t)
   #D=(v1t+Bt-2x1+2A)/t^3                    # from C=C
@@ -106,10 +117,22 @@ sub ABCD {
   return($A,$B,$C,$D);
 }
 
-sub time {
+sub pointlist {
+  my $self=shift();
+  my @list=@_;
+  @list=@{$self->timelist()} if (scalar(@list)== 0);
+  my @points=();
+  foreach (@list) {
+    push @points, $self->point($_);
+  }
+  return \@points;
+}
+
+sub point {
   my $self=shift();
   my $timereal=shift();
-  my $t=$timereal-$self->{'pt1'}->{'time'};
+  my @point=();
+  my $t=$timereal-$self->{'pt0'}->{'time'};
   my ($Alat, $Blat, $Clat, $Dlat)=($self->{'Alat'}, $self->{'Blat'},$self->{'Clat'},$self->{'Dlat'});
   my ($Alon, $Blon, $Clon, $Dlon)=($self->{'Alon'}, $self->{'Blon'},$self->{'Clon'},$self->{'Dlon'});
   my $lat=$Alat + $Blat * $t + $Clat * $t ** 2 + $Dlat * $t ** 3;
@@ -129,30 +152,42 @@ sub time {
           heading=>$heading};
 }
 
-sub list {
+sub timelist {
   my $self=shift();
-  my $t0=$self->{'pt1'}->{'time'};
-  my $t1=$self->{'pt2'}->{'time'};
+  my $t0=$self->{'pt0'}->{'time'};
+  my $t1=$self->{'pt1'}->{'time'};
   my $dt=$t1-$t0;
-  my $count=shift() || int($dt + 1.5);
+  my $count=shift() || round($dt);
   my @list;
   foreach(0..$count) {
     my $t=$t0+$dt*($_/$count); 
-    push @list, $self->time($t);
+    push @list, $t;
   }
   return \@list;
 }
 
+sub round {
+  return int(shift() + 0.5);
+}
+
 1;
 
 __END__
 
 =head1 TODO
 
+Integrate a better lat,Lon to meter conversions.
+
+Migrate certain hash reference variables to object methods.
+
+Add a timeref method
+
 =head1 BUGS
 
 =head1 LIMITS
 
+I use a very rough conversion from degrees to meters and then back.  It is accurate for short distances.
+
 =head1 AUTHOR
 
 Michael R. Davis qw/perl michaelrdavis com/
